# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['paas_service',
 'paas_service.auth',
 'paas_service.management',
 'paas_service.management.commands',
 'paas_service.migrations']

package_data = \
{'': ['*']}

install_requires = \
['blue-krill>=1.0.15',
 'django-translated-fields',
 'jsonfield',
 'pyjwt>=1.6.4,<2.0.0']

setup_kwargs = {
    'name': 'paas-service',
    'version': '1.1.5',
    'description': 'A Django application for developing BK-PaaS add-on services.',
    'long_description': '# Paas Service\n\n\xe8\x93\x9d\xe9\xb2\xb8 PaaS \xe5\xb9\xb3\xe5\x8f\xb0\xe5\xa2\x9e\xe5\xbc\xba\xe6\x9c\x8d\xe5\x8a\xa1\xe6\xa1\x86\xe6\x9e\xb6\n\n\n## \xe7\x89\x88\xe6\x9c\xac\xe5\x8e\x86\xe5\x8f\xb2\n\n\xe8\xaf\xa6\xe8\xa7\x81 `CHANGES.md`\xe3\x80\x82\n\n## \xe4\xbd\xbf\xe7\x94\xa8\xe6\x8c\x87\xe5\x8d\x97\n\n1. \xe6\x9b\xb4\xe6\x96\xb0 settings\xef\xbc\x9a\n```python\nINSTALLED_APPS = [\n    ...\n    \'paas_service\',\n    ...\n]\n\nMIDDLEWARE = [\n    ...\n    \'paas_service.auth.middleware.VerifiedClientMiddleware\',\n    ...\n]\n\n# \xe6\x95\xb0\xe6\x8d\xae\xe5\xba\x93\xe6\x95\x8f\xe6\x84\x9f\xe5\xad\x97\xe6\xae\xb5\xe5\x8a\xa0\xe5\xaf\x86 Key\nBKKRILL_ENCRYPT_SECRET_KEY = base64.b64encode(b\'\\x01\' * 32)\n\n# \xe4\xb8\x8e PaaS \xe5\xb9\xb3\xe5\x8f\xb0\xe9\x80\x9a\xe4\xbf\xa1\xe7\x9a\x84 JWT \xe4\xbf\xa1\xe6\x81\xaf\nPAAS_SERVICE_JWT_CLIENTS = [\n    {\n        "iss": "paas-v3",\n        "key": "123..........",\n        "algorithm": "HS256",\n    },\n]\n\n# \xe5\xa2\x9e\xe5\xbc\xba\xe6\x9c\x8d\xe5\x8a\xa1\xe4\xbe\x9b\xe5\xba\x94\xe5\x95\x86\xe7\xb1\xbb\nPAAS_SERVICE_PROVIDER_CLS = "svc_xxx.vendor.provider.Provider"\n# \xe5\xa2\x9e\xe5\xbc\xba\xe6\x9c\x8d\xe5\x8a\xa1\xe5\xae\x9e\xe4\xbe\x8b\xe4\xbf\xa1\xe6\x81\xaf\xe6\xb8\xb2\xe6\x9f\x93\xe5\x87\xbd\xe6\x95\xb0\nPAAS_SERVICE_SVC_INSTANCE_RENDER_FUNC = "svc_xxx.vendor.render.render_instance_data"\n\n# \xe8\xae\xbe\xe7\xbd\xae\xe8\xaf\xad\xe8\xa8\x80\xef\xbc\x8c\xe6\xb3\xa8\xe6\x84\x8f\xef\xbc\x9a\xe7\x9b\xae\xe5\x89\x8d\xe5\x9b\xbd\xe9\x99\x85\xe5\x8c\x96\xe5\x8f\xaa\xe6\x94\xaf\xe6\x8c\x81: \xe7\xae\x80\xe4\xbd\x93\xe4\xb8\xad\xe6\x96\x87 \xe5\x92\x8c English\nLANGUAGE_CODE = \'zh-cn\'\n\nLANGUAGES = [("zh-cn", "\xe7\xae\x80\xe4\xbd\x93\xe4\xb8\xad\xe6\x96\x87"), ("en", "English")]\n```\n\n2. \xe5\x8d\x95\xe5\x85\x83\xe6\xb5\x8b\xe8\xaf\x95\n\n\xe9\xa6\x96\xe5\x85\x88\xef\xbc\x8c\xe5\xae\x89\xe8\xa3\x85 pytest\xe3\x80\x81pytest-django\xe3\x80\x82\n\n\xe7\x84\xb6\xe5\x90\x8e\xe6\x89\xa7\xe8\xa1\x8c `make test` \xe8\xbf\x90\xe8\xa1\x8c\xe6\x89\x80\xe6\x9c\x89\xe5\x8d\x95\xe5\x85\x83\xe6\xb5\x8b\xe8\xaf\x95\xe3\x80\x82\n',
    'author': 'blueking',
    'author_email': 'blueking@tencent.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.2,<3.11',
}


setup(**setup_kwargs)
